#include <stdlib.h>
#include <string>
#include <fstream>
#include <iostream>
#include "tree.hh" //Please see license in tree.hh.
using namespace std;

#define DEBUG_MODE false
#define MAXLEN 75
#define DELIM '|'


int instr( char searchme[], char find ) {
	int len; len = strlen(searchme);
	for (int x=0; x<len; x++) {
		if ( searchme[x] == find ) return(x);
	}
	return(-1);
}

int main(int argc, char *argv[]) {
	char *strInputFile;

	if (argc != 2) {
		if (DEBUG_MODE) {
			cerr<<"argc: "<<argc<<endl;
			for (int x=0; x<argc; x++)
				cerr<<"argv["<<x<<"]: "<<argv[x]<<endl;
		}
		cerr<<"Usage:\t"<<argv[0]<<" input_file"<<endl;
		cerr<<"\tinput_file: path to the switchdata input file."<<endl;
		cerr<<endl;
		return(-1);
	}

	// Parse parameters.
	strInputFile = (char *) calloc ( strlen(argv[1]), sizeof(char) );
	strncpy( strInputFile, argv[1], strlen(argv[1]) );
	if (DEBUG_MODE) { //Parameters
		cerr<<"Input File: ["<<strInputFile<<"]"<<endl;
	}

	ifstream infile (strInputFile, ios::in);
	if ( ! infile ) {
		cerr<<"Cannot open input file: "<<strInputFile<<endl;
		return(-1);
	}

	char *strLine; strLine = (char *) calloc ( MAXLEN, sizeof(char) );
	tree<string> tr;
	tree<string>::iterator top, parent, child, loc, root;
	top=tr.begin();
	root=tr.insert( top, "ROOT_NODE" );

	while (infile.getline( strLine, MAXLEN, '\n' )) {
		if (DEBUG_MODE) cerr<<strLine<<endl;
		// everything (if any) before the DELIM is the parent
		// everything after the DELIM is the child

		char strParent[MAXLEN], strChild[MAXLEN];
		int delim_pos; delim_pos = instr( strLine, DELIM );
		if (delim_pos != -1) { //found a delimeter
			strncpy( strParent, "\0", MAXLEN ); 
			strncpy( strChild, "\0", MAXLEN );
			if (DEBUG_MODE) cerr<<"Delimiter ("<<DELIM<<") found at: "<<delim_pos<<endl;
			if (delim_pos == 0) {
				strncpy( strParent, "ROOT_NODE", MAXLEN );
				strncpy( strChild, strncat( &(strLine[1]), "\0", MAXLEN ), MAXLEN );
				if ( int(strChild[ strlen(strChild)-1 ]) == 13 ) strChild[ strlen(strChild)-1 ] = '\0';
				tr.append_child( root, strChild );
			}
			else {
				strncpy( strParent, strLine, delim_pos);
				strncpy( strChild, strncat( &(strLine[delim_pos+1]), "\0", MAXLEN ), MAXLEN );
				if ( int(strChild[ strlen(strChild)-1 ]) == 13 ) strChild[ strlen(strChild)-1 ] = '\0';
				parent = find(tr.begin(), tr.end(), strParent);
				tr.append_child( parent, strChild );
			}
			if (DEBUG_MODE) {
				for ( int z = 0; z < strlen(strParent); z++ )
					cerr<<"strParent["<<z<<"]: ["<<strParent[z]<<"] ("<<int(strParent[z])<<")"<<endl;
				for ( int z = 0; z < strlen(strChild); z++ )
					cerr<<"strChild["<<z<<"]:  ["<<strChild[z]<<"] ("<<int(strChild[z])<<")"<<endl;
				cerr<<"Parent Node Name: ["<<strParent<<"]"<<endl;
				cerr<<"Child Node Name:  ["<<strChild<<"]"<<endl;
			}
		}
	}

	infile.close();

	cout<<"-----------------------"<<endl;
	loc=find(tr.begin(), tr.end(), "ROOT_NODE");
	if(loc!=tr.end()) {
		tree<string>::iterator sib2=tr.begin(loc);
		tree<string>::iterator end2=tr.end(loc);
		while(sib2!=end2) {
			for(int i=0; i<tr.depth(sib2)-1; ++i) 
				cout << "    ";
			cout << (*sib2) << endl;
			++sib2;
		}
	}
	else
		cout<<"ROOT NODE NOT FOUND"<<endl;
	cout<<"-----------------------"<<endl;

	free(strInputFile); free(strLine);
	return(0);
}
